@@ -4,15 +4,13 @@
 const { Clutter, Gio, GLib, GObject, NM, Pango, Shell, St } = imports.gi;
 const Signals = imports.signals;
 
+const Config = imports.misc.config;
 const Dialog = imports.ui.dialog;
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
 const ModalDialog = imports.ui.modalDialog;
 const ShellEntry = imports.ui.shellEntry;
 
-Gio._promisify(Shell.NetworkAgent.prototype,
-    'search_vpn_plugin', 'search_vpn_plugin_finish');
-
 const VPN_UI_GROUP = 'VPN Plugin UI';
 
 var NetworkSecretDialog = GObject.registerClass(
@@ -58,7 +56,7 @@ class NetworkSecretDialog extends ModalDialog.ModalDialog {
 
             secret.entry = new St.Entry({ style_class: 'prompt-dialog-password-entry',
                                           text: secret.value, can_focus: reactive,
-                                          reactive: reactive,
+                                          reactive,
                                           x_expand: true });
             ShellEntry.addContextMenu(secret.entry,
                                       { isPassword: secret.password });
@@ -108,10 +106,7 @@ class NetworkSecretDialog extends ModalDialog.ModalDialog {
             descriptionLabel.clutter_text.line_wrap = true;
             descriptionLabel.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
 
-            contentBox.messageBox.add(descriptionLabel,
-                                      { y_fill: true,
-                                        y_align: St.Align.START,
-                                        expand: true });
+            contentBox.messageBox.add_child(descriptionLabel);
         }
 
         this._okButton = {
@@ -174,7 +169,7 @@ class NetworkSecretDialog extends ModalDialog.ModalDialog {
             return true;
         }
 
-        return (value.length >= 8 && value.length <= 63);
+        return value.length >= 8 && value.length <= 63;
     }
 
     _validateStaticWep(secret) {
@@ -227,11 +222,12 @@ class NetworkSecretDialog extends ModalDialog.ModalDialog {
                            validate: this._validateStaticWep, password: true });
             break;
         case 'ieee8021x':
-            if (wirelessSecuritySetting.auth_alg == 'leap') // Cisco LEAP
+            if (wirelessSecuritySetting.auth_alg == 'leap') { // Cisco LEAP
                 secrets.push({ label: _("Password: "), key: 'leap-password',
                                value: wirelessSecuritySetting.leap_password || '', password: true });
-            else // Dynamic (IEEE 802.1x) WEP
+            } else { // Dynamic (IEEE 802.1x) WEP
                 this._get8021xSecrets(secrets);
+            }
             break;
         case 'wpa-eap':
             this._get8021xSecrets(secrets);
@@ -246,15 +242,18 @@ class NetworkSecretDialog extends ModalDialog.ModalDialog {
 
         /* If hints were given we know exactly what we need to ask */
         if (this._settingName == "802-1x" && this._hints.length) {
-            if (this._hints.includes('identity'))
+            if (this._hints.includes('identity')) {
                 secrets.push({ label: _("Username: "), key: 'identity',
                                value: ieee8021xSetting.identity || '', password: false });
-            if (this._hints.includes('password'))
+            }
+            if (this._hints.includes('password')) {
                 secrets.push({ label: _("Password: "), key: 'password',
                                value: ieee8021xSetting.password || '', password: true });
-            if (this._hints.includes('private-key-password'))
+            }
+            if (this._hints.includes('private-key-password')) {
                 secrets.push({ label: _("Private key password: "), key: 'private-key-password',
                                value: ieee8021xSetting.private_key_password || '', password: true });
+            }
             return;
         }
 
@@ -558,7 +557,7 @@ var VPNRequestHandler = class {
                     contentOverride.secrets.push({
                         label: keyfile.get_string(groups[i], 'Label'),
                         key: groups[i],
-                        value: value,
+                        value,
                         password: keyfile.get_boolean(groups[i], 'IsSecret'),
                     });
                 } else {
@@ -624,6 +623,14 @@ var NetworkAgent = class {
         this._vpnRequests = { };
         this._notifications = { };
 
+        this._pluginDir = Gio.file_new_for_path(Config.VPNDIR);
+        try {
+            let monitor = this._pluginDir.monitor(Gio.FileMonitorFlags.NONE, null);
+            monitor.connect('changed', () => (this._vpnCacheBuilt = false));
+        } catch (e) {
+            log(`Failed to create monitor for VPN plugin dir: ${e.message}`);
+        }
+
         this._native.connect('new-request', this._newRequest.bind(this));
         this._native.connect('cancel-request', this._cancelRequest.bind(this));
 
@@ -728,7 +735,7 @@ var NetworkAgent = class {
         });
 
         Main.messageTray.add(source);
-        source.notify(notification);
+        source.showNotification(notification);
     }
 
     _newRequest(agent, requestId, connection, settingName, hints, flags) {
@@ -763,11 +770,13 @@ var NetworkAgent = class {
         }
     }
 
-    async _vpnRequest(requestId, connection, hints, flags) {
+    _vpnRequest(requestId, connection, hints, flags) {
         let vpnSetting = connection.get_setting_vpn();
         let serviceType = vpnSetting.service_type;
 
-        let binary = await this._findAuthBinary(serviceType);
+        this._buildVPNServiceCache();
+
+        let binary = this._vpnBinaries[serviceType];
         if (!binary) {
             log('Invalid VPN service type (cannot find authentication binary)');
 
@@ -783,30 +792,36 @@ var NetworkAgent = class {
         this._vpnRequests[requestId] = vpnRequest;
     }
 
-    async _findAuthBinary(serviceType) {
-        let plugin;
+    _buildVPNServiceCache() {
+        if (this._vpnCacheBuilt)
+            return;
 
-        try {
-            plugin = await this._native.search_vpn_plugin(serviceType);
-        } catch (e) {
-            logError(e);
-            return null;
-        }
+        this._vpnCacheBuilt = true;
+        this._vpnBinaries = { };
 
-        const fileName = plugin.get_auth_dialog();
-        if (!GLib.file_test(fileName, GLib.FileTest.IS_EXECUTABLE)) {
-            log('VPN plugin at %s is not executable'.format(fileName));
-            return null;
-        }
+        NM.VpnPluginInfo.list_load().forEach(plugin => {
+            let service = plugin.get_service();
+            let fileName = plugin.get_auth_dialog();
+            let supportsHints = plugin.supports_hints();
+            let externalUIMode = false;
 
-        const prop = plugin.lookup_property('GNOME', 'supports-external-ui-mode');
-        const trimmedProp = prop ? prop.trim().toLowerCase() : '';
+            let prop = plugin.lookup_property('GNOME', 'supports-external-ui-mode');
+            if (prop) {
+                prop = prop.trim().toLowerCase();
+                externalUIMode = ['true', 'yes', 'on', '1'].includes(prop);
+            }
 
-        return {
-            fileName,
-            supportsHints: plugin.supports_hints(),
-            externalUIMode: ['true', 'yes', 'on', '1'].includes(trimmedProp),
-        };
+            if (GLib.file_test(fileName, GLib.FileTest.IS_EXECUTABLE)) {
+                let binary = { fileName, externalUIMode, supportsHints };
+                this._vpnBinaries[service] = binary;
+
+                plugin.get_aliases().forEach(alias => {
+                    this._vpnBinaries[alias] = binary;
+                });
+            } else {
+                log('VPN plugin at %s is not executable'.format(fileName));
+            }
+        });
     }
 };
 var Component = NetworkAgent;
