@@ -36,31 +36,21 @@ class Dialog extends St.Widget {
         this._dialog.request_mode = Clutter.RequestMode.HEIGHT_FOR_WIDTH;
         this._dialog.set_offscreen_redirect(Clutter.OffscreenRedirect.ALWAYS);
 
-        this.contentLayout = new St.BoxLayout({ vertical: true,
-                                                style_class: "modal-dialog-content-box" });
-        this._dialog.add(this.contentLayout,
-                         { expand: true,
-                           x_fill: true,
-                           y_fill: true,
-                           x_align: St.Align.MIDDLE,
-                           y_align: St.Align.START });
+        this.contentLayout = new St.BoxLayout({
+            vertical: true,
+            style_class: 'modal-dialog-content-box',
+            y_expand: true,
+        });
+        this._dialog.add_child(this.contentLayout);
 
-        this.buttonLayout = new St.Widget ({ layout_manager: new Clutter.BoxLayout({ homogeneous: true }) });
-        this._dialog.add(this.buttonLayout,
-                         { x_align: St.Align.MIDDLE,
-                           y_align: St.Align.START });
-    }
-
-    makeInactive() {
-        if (this._eventId != 0)
-            this._parentActor.disconnect(this._eventId);
-        this._eventId = 0;
-
-        this.buttonLayout.get_children().forEach(c => c.set_reactive(false));
+        this.buttonLayout = new St.Widget({ layout_manager: new Clutter.BoxLayout({ homogeneous: true }) });
+        this._dialog.add_child(this.buttonLayout);
     }
 
     _onDestroy() {
-        this.makeInactive();
+        if (this._eventId != 0)
+            this._parentActor.disconnect(this._eventId);
+        this._eventId = 0;
     }
 
     _modalEventHandler(actor, event) {
@@ -106,7 +96,7 @@ class Dialog extends St.Widget {
     }
 
     addContent(actor) {
-        this.contentLayout.add (actor, { expand: true });
+        this.contentLayout.add(actor, { expand: true });
     }
 
     addButton(buttonInfo) {
@@ -127,7 +117,7 @@ class Dialog extends St.Widget {
                                      can_focus: true,
                                      x_expand: true,
                                      y_expand: true,
-                                     label: label });
+                                     label });
         button.connect('clicked', action);
 
         buttonInfo['button'] = button;
@@ -169,8 +159,8 @@ var MessageDialogContent = GObject.registerClass({
         'body': GObject.ParamSpec.string('body', 'body', 'body',
                                          GObject.ParamFlags.READWRITE |
                                          GObject.ParamFlags.CONSTRUCT,
-                                         null)
-    }
+                                         null),
+    },
 }, class MessageDialogContent extends St.BoxLayout {
     _init(params) {
         this._icon = new St.Icon({ y_align: Clutter.ActorAlign.START });
@@ -221,7 +211,7 @@ var MessageDialogContent = GObject.registerClass({
     set icon(icon) {
         this._icon.set({
             gicon: icon,
-            visible: icon != null
+            visible: icon != null,
         });
         this.notify('icon');
     }
@@ -241,7 +231,7 @@ var MessageDialogContent = GObject.registerClass({
     _setLabel(label, prop, value) {
         label.set({
             text: value || '',
-            visible: value != null
+            visible: value != null,
         });
         this.notify(prop);
     }
