@@ -2,7 +2,6 @@
 /* exported ShellMountOperation, GnomeShellMountOpHandler */
 
 const { Clutter, Gio, GLib, GObject, Pango, Shell, St } = imports.gi;
-const Signals = imports.signals;
 
 const Animation = imports.ui.animation;
 const CheckBox = imports.ui.checkBox;
@@ -44,18 +43,19 @@ function _setLabelsForMessage(content, message) {
 
 /* -------------------------------------------------------- */
 
-var ListItem = class {
-    constructor(app) {
-        this._app = app;
-
+var ListItem = GObject.registerClass({
+    Signals: { 'activate': {} },
+}, class ListItem extends St.Button {
+    _init(app) {
         let layout = new St.BoxLayout({ vertical: false });
+        super._init({
+            style_class: 'mount-dialog-app-list-item',
+            can_focus: true,
+            child: layout,
+            reactive: true,
+        });
 
-        this.actor = new St.Button({ style_class: 'mount-dialog-app-list-item',
-                                     can_focus: true,
-                                     child: layout,
-                                     reactive: true,
-                                     x_align: St.Align.START,
-                                     x_fill: true });
+        this._app = app;
 
         this._icon = this._app.create_icon_texture(LIST_ITEM_ICON_SIZE);
 
@@ -63,21 +63,20 @@ var ListItem = class {
                                    child: this._icon });
         layout.add(iconBin);
 
-        this._nameLabel = new St.Label({ text: this._app.get_name(),
-                                         style_class: 'mount-dialog-app-list-item-name' });
-        let labelBin = new St.Bin({ y_align: St.Align.MIDDLE,
-                                    child: this._nameLabel });
+        this._nameLabel = new St.Label({
+            text: this._app.get_name(),
+            style_class: 'mount-dialog-app-list-item-name',
+            y_align: Clutter.ActorAlign.CENTER,
+        });
+        let labelBin = new St.Bin({ child: this._nameLabel });
         layout.add(labelBin);
-
-        this.actor.connect('clicked', this._onClicked.bind(this));
     }
 
-    _onClicked() {
+    vfunc_clicked() {
         this.emit('activate');
         this._app.activate();
     }
-};
-Signals.addSignalMethods(ListItem.prototype);
+});
 
 var ShellMountOperation = class {
     constructor(source, params) {
@@ -219,9 +218,10 @@ var ShellMountOperation = class {
     }
 };
 
-var ShellUnmountNotifier = class extends MessageTray.Source {
-    constructor() {
-        super('', 'media-removable');
+var ShellUnmountNotifier = GObject.registerClass(
+class ShellUnmountNotifier extends MessageTray.Source {
+    _init() {
+        super._init('', 'media-removable');
 
         this._notification = null;
         Main.messageTray.add(this);
@@ -238,7 +238,7 @@ var ShellUnmountNotifier = class extends MessageTray.Source {
             this._notification.update(header, text);
         }
 
-        this.notify(this._notification);
+        this.showNotification(this._notification);
     }
 
     done(message) {
@@ -251,19 +251,19 @@ var ShellUnmountNotifier = class extends MessageTray.Source {
             let notification = new MessageTray.Notification(this, message, null);
             notification.setTransient(true);
 
-            this.notify(notification);
+            this.showNotification(notification);
         }
     }
-};
+});
 
 var ShellMountQuestionDialog = GObject.registerClass({
-    Signals: { 'response': { param_types: [GObject.TYPE_INT] } }
+    Signals: { 'response': { param_types: [GObject.TYPE_INT] } },
 }, class ShellMountQuestionDialog extends ModalDialog.ModalDialog {
     _init(icon) {
         super._init({ styleClass: 'mount-dialog' });
 
         this._content = new Dialog.MessageDialogContent({ icon });
-        this.contentLayout.add(this._content, { x_fill: true, y_fill: false });
+        this.contentLayout.add_child(this._content);
     }
 
     update(message, choices) {
@@ -278,7 +278,7 @@ var ShellMountPasswordDialog = GObject.registerClass({
                                            GObject.TYPE_BOOLEAN,
                                            GObject.TYPE_BOOLEAN,
                                            GObject.TYPE_BOOLEAN,
-                                           GObject.TYPE_UINT] } }
+                                           GObject.TYPE_UINT] } },
 }, class ShellMountPasswordDialog extends ModalDialog.ModalDialog {
     _init(message, icon, flags) {
         let strings = message.split('\n');
@@ -299,18 +299,21 @@ var ShellMountPasswordDialog = GObject.registerClass({
         let rtl = grid.get_text_direction() === Clutter.TextDirection.RTL;
 
         if (flags & Gio.AskPasswordFlags.TCRYPT) {
-            this._keyfilesLabel = new St.Label(({ style_class: 'prompt-dialog-keyfiles-label',
-                                                  visible: false }));
+            this._keyfilesLabel = new St.Label({
+                style_class: 'prompt-dialog-keyfiles-label',
+                visible: false,
+                y_align: Clutter.ActorAlign.CENTER,
+            });
 
             this._hiddenVolume = new CheckBox.CheckBox(_("Hidden Volume"));
-            content.messageBox.add(this._hiddenVolume.actor);
+            content.messageBox.add(this._hiddenVolume);
 
             this._systemVolume = new CheckBox.CheckBox(_("Windows System Volume"));
-            content.messageBox.add(this._systemVolume.actor);
+            content.messageBox.add(this._systemVolume);
 
             this._keyfilesCheckbox = new CheckBox.CheckBox(_("Uses Keyfiles"));
-            this._keyfilesCheckbox.actor.connect("clicked", this._onKeyfilesCheckboxClicked.bind(this));
-            content.messageBox.add(this._keyfilesCheckbox.actor);
+            this._keyfilesCheckbox.connect("clicked", this._onKeyfilesCheckboxClicked.bind(this));
+            content.messageBox.add(this._keyfilesCheckbox);
 
             this._keyfilesLabel.clutter_text.set_markup(
                 /* Translators: %s is the Disks application */
@@ -318,7 +321,7 @@ var ShellMountPasswordDialog = GObject.registerClass({
             );
             this._keyfilesLabel.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
             this._keyfilesLabel.clutter_text.line_wrap = true;
-            content.messageBox.add(this._keyfilesLabel, { y_fill: false, y_align: St.Align.MIDDLE, expand: true });
+            content.messageBox.add_child(this._keyfilesLabel);
 
             this._pimLabel = new St.Label({ style_class: 'prompt-dialog-password-label',
                                             text: _("PIM Number"),
@@ -359,8 +362,10 @@ var ShellMountPasswordDialog = GObject.registerClass({
         this._passwordEntry.clutter_text.set_password_char('\u25cf'); // ● U+25CF BLACK CIRCLE
         ShellEntry.addContextMenu(this._passwordEntry, { isPassword: true });
         this.setInitialKeyFocus(this._passwordEntry);
-        this._workSpinner = new Animation.Spinner(WORK_SPINNER_ICON_SIZE, true);
-        this._passwordEntry.secondary_icon = this._workSpinner.actor;
+        this._workSpinner = new Animation.Spinner(WORK_SPINNER_ICON_SIZE, {
+            animate: true,
+        });
+        this._passwordEntry.secondary_icon = this._workSpinner;
 
         if (rtl) {
             layout.attach(this._passwordEntry, 0, 1, 1, 1);
@@ -381,9 +386,9 @@ var ShellMountPasswordDialog = GObject.registerClass({
 
         if (flags & Gio.AskPasswordFlags.SAVING_SUPPORTED) {
             this._rememberChoice = new CheckBox.CheckBox(_("Remember Password"));
-            this._rememberChoice.actor.checked =
+            this._rememberChoice.checked =
                 global.settings.get_boolean(REMEMBER_MOUNT_PASSWORD_KEY);
-            content.messageBox.add(this._rememberChoice.actor);
+            content.messageBox.add(this._rememberChoice);
         } else {
             this._rememberChoice = null;
         }
@@ -391,7 +396,7 @@ var ShellMountPasswordDialog = GObject.registerClass({
         this._defaultButtons = [{
             label: _("Cancel"),
             action: this._onCancelButton.bind(this),
-            key: Clutter.Escape,
+            key: Clutter.KEY_Escape,
         }, {
             label: _("Unlock"),
             action: this._onUnlockButton.bind(this),
@@ -401,7 +406,7 @@ var ShellMountPasswordDialog = GObject.registerClass({
         this._usesKeyfilesButtons = [{
             label: _("Cancel"),
             action: this._onCancelButton.bind(this),
-            key: Clutter.Escape,
+            key: Clutter.KEY_Escape,
         }, {
             /* Translators: %s is the Disks application */
             label: _("Open %s").format(disksApp.get_name()),
@@ -439,22 +444,22 @@ var ShellMountPasswordDialog = GObject.registerClass({
         }
 
         global.settings.set_boolean(REMEMBER_MOUNT_PASSWORD_KEY,
-            this._rememberChoice && this._rememberChoice.actor.checked);
+            this._rememberChoice && this._rememberChoice.checked);
 
         this._workSpinner.play();
         this.emit('response', 1,
             this._passwordEntry.get_text(),
             this._rememberChoice &&
-            this._rememberChoice.actor.checked,
+            this._rememberChoice.checked,
             this._hiddenVolume &&
-            this._hiddenVolume.actor.checked,
+            this._hiddenVolume.checked,
             this._systemVolume &&
-            this._systemVolume.actor.checked,
+            this._systemVolume.checked,
             parseInt(pim));
     }
 
     _onKeyfilesCheckboxClicked() {
-        let useKeyfiles = this._keyfilesCheckbox.actor.checked;
+        let useKeyfiles = this._keyfilesCheckbox.checked;
         this._passwordEntry.reactive = !useKeyfiles;
         this._passwordEntry.can_focus = !useKeyfiles;
         this._passwordEntry.clutter_text.editable = !useKeyfiles;
@@ -463,42 +468,45 @@ var ShellMountPasswordDialog = GObject.registerClass({
         this._pimEntry.can_focus = !useKeyfiles;
         this._pimEntry.clutter_text.editable = !useKeyfiles;
         this._pimEntry.clutter_text.selectable = !useKeyfiles;
-        this._rememberChoice.actor.reactive = !useKeyfiles;
-        this._rememberChoice.actor.can_focus = !useKeyfiles;
+        this._rememberChoice.reactive = !useKeyfiles;
+        this._rememberChoice.can_focus = !useKeyfiles;
         this._keyfilesLabel.visible = useKeyfiles;
         this.setButtons(useKeyfiles ? this._usesKeyfilesButtons : this._defaultButtons);
     }
 
     _onOpenDisksButton() {
         let app = Shell.AppSystem.get_default().lookup_app('org.gnome.DiskUtility.desktop');
-        if (app)
+        if (app) {
             app.activate();
-        else
+        } else {
             Main.notifyError(
                 /* Translators: %s is the Disks application */
                 _("Unable to start %s").format(app.get_name()),
                 /* Translators: %s is the Disks application */
                 _("Couldn’t find the %s application").format(app.get_name())
             );
+        }
         this._onCancelButton();
     }
 });
 
 var ShellProcessesDialog = GObject.registerClass({
-    Signals: { 'response': { param_types: [GObject.TYPE_INT] } }
+    Signals: { 'response': { param_types: [GObject.TYPE_INT] } },
 }, class ShellProcessesDialog extends ModalDialog.ModalDialog {
     _init(icon) {
         super._init({ styleClass: 'mount-dialog' });
 
         this._content = new Dialog.MessageDialogContent({ icon });
-        this.contentLayout.add(this._content, { x_fill: true, y_fill: false });
+        this.contentLayout.add_child(this._content);
 
-        let scrollView = new St.ScrollView({ style_class: 'mount-dialog-app-list' });
+        let scrollView = new St.ScrollView({
+            style_class: 'mount-dialog-app-list',
+            x_expand: true,
+            y_expand: true,
+        });
         scrollView.set_policy(St.PolicyType.NEVER,
                               St.PolicyType.AUTOMATIC);
-        this.contentLayout.add(scrollView,
-                               { x_fill: true,
-                                 y_fill: true });
+        this.contentLayout.add_child(scrollView);
         scrollView.hide();
 
         this._applicationList = new St.BoxLayout({ vertical: true });
@@ -527,7 +535,7 @@ var ShellProcessesDialog = GObject.registerClass({
                 return;
 
             let item = new ListItem(app);
-            this._applicationList.add(item.actor, { x_fill: true });
+            this._applicationList.add_child(item);
 
             item.connect('activate', () => {
                 // use -1 to indicate Cancel
@@ -549,7 +557,7 @@ var ShellMountOperationType = {
     NONE: 0,
     ASK_PASSWORD: 1,
     ASK_QUESTION: 2,
-    SHOW_PROCESSES: 3
+    SHOW_PROCESSES: 3,
 };
 
 var GnomeShellMountOpHandler = class {
@@ -612,18 +620,22 @@ var GnomeShellMountOpHandler = class {
 
     /**
      * AskPassword:
-     * @id: an opaque ID identifying the object for which the operation is requested
-     *      The ID must be unique in the context of the calling process.
-     * @message: the message to display
-     * @icon_name: the name of an icon to display
-     * @default_user: the default username for display
-     * @default_domain: the default domain for display
-     * @flags: a set of GAskPasswordFlags
-     * @response: a GMountOperationResult
-     * @response_details: a dictionary containing the response details as
-     * entered by the user. The dictionary MAY contain the following properties:
+     * @param {Array} params
+     *   {string} id: an opaque ID identifying the object for which
+     *       the operation is requested
+     *   {string} message: the message to display
+     *   {string} icon_name: the name of an icon to display
+     *   {string} default_user: the default username for display
+     *   {string} default_domain: the default domain for display
+     *   {Gio.AskPasswordFlags} flags: a set of GAskPasswordFlags
+     *   {Gio.MountOperationResults} response: a GMountOperationResult
+     *   {Object} response_details: a dictionary containing response details as
+     *       entered by the user. The dictionary MAY contain the following
+     *       properties:
      *   - "password" -> (s): a password to be used to complete the mount operation
      *   - "password_save" -> (u): a GPasswordSave
+     * @param {Gio.DBusMethodInvocation} invocation
+     *      The ID must be unique in the context of the calling process.
      *
      * The dialog will stay visible until clients call the Close() method, or
      * another dialog becomes visible.
@@ -667,16 +679,14 @@ var GnomeShellMountOpHandler = class {
 
     /**
      * AskQuestion:
-     * @id: an opaque ID identifying the object for which the operation is requested
+     * @param {Array} params - params
+     *   {string} id: an opaque ID identifying the object for which
+     *       the operation is requested
      *      The ID must be unique in the context of the calling process.
-     * @message: the message to display
-     * @icon_name: the name of an icon to display
-     * @choices: an array of choice strings
-     * GetResponse:
-     * @response: a GMountOperationResult
-     * @response_details: a dictionary containing the response details as
-     * entered by the user. The dictionary MAY contain the following properties:
-     *   - "choice" -> (i): the chosen answer among the array of strings passed in
+     *   {string} message: the message to display
+     *   {string} icon_name: the name of an icon to display
+     *   {string[]} choices: an array of choice strings
+     * @param {Gio.DBusMethodInvocation} invocation - invocation
      *
      * The dialog will stay visible until clients call the Close() method, or
      * another dialog becomes visible.
@@ -705,16 +715,15 @@ var GnomeShellMountOpHandler = class {
 
     /**
      * ShowProcesses:
-     * @id: an opaque ID identifying the object for which the operation is requested
+     * @param {Array} params - params
+     *   {string} id: an opaque ID identifying the object for which
+     *       the operation is requested
      *      The ID must be unique in the context of the calling process.
-     * @message: the message to display
-     * @icon_name: the name of an icon to display
-     * @application_pids: the PIDs of the applications to display
-     * @choices: an array of choice strings
-     * @response: a GMountOperationResult
-     * @response_details: a dictionary containing the response details as
-     * entered by the user. The dictionary MAY contain the following properties:
-     *   - "choice" -> (i): the chosen answer among the array of strings passed in
+     *   {string} message: the message to display
+     *   {string} icon_name: the name of an icon to display
+     *   {number[]} application_pids: the PIDs of the applications to display
+     *   {string[]} choices: an array of choice strings
+     * @param {Gio.DBusMethodInvocation} invocation - invocation
      *
      * The dialog will stay visible until clients call the Close() method, or
      * another dialog becomes visible.
@@ -753,6 +762,8 @@ var GnomeShellMountOpHandler = class {
 
     /**
      * Close:
+     * @param {Array} _params - params
+     * @param {Gio.DBusMethodInvocation} _invocation - invocation
      *
      * Closes a dialog previously opened by AskPassword, AskQuestion or ShowProcesses.
      * If no dialog is open, does nothing.
