@@ -30,7 +30,8 @@ var WINDOW_ANIMATION_TIME = 250;
 var DIM_BRIGHTNESS = -0.3;
 var DIM_TIME = 500;
 var UNDIM_TIME = 250;
-var MOTION_THRESHOLD = 100;
+var WS_MOTION_THRESHOLD = 100;
+var APP_MOTION_THRESHOLD = 30;
 
 var ONE_SECOND = 1000; // in ms
 
@@ -40,6 +41,8 @@ const GSD_WACOM_OBJECT_PATH = '/org/gnome/SettingsDaemon/Wacom';
 const GsdWacomIface = loadInterfaceXML('org.gnome.SettingsDaemon.Wacom');
 const GsdWacomProxy = Gio.DBusProxy.makeProxyWrapper(GsdWacomIface);
 
+const WINDOW_DIMMER_EFFECT_NAME = "gnome-shell-window-dimmer";
+
 var DisplayChangeDialog = GObject.registerClass(
 class DisplayChangeDialog extends ModalDialog.ModalDialog {
     _init(wm) {
@@ -54,18 +57,20 @@ class DisplayChangeDialog extends ModalDialog.ModalDialog {
         let title = _("Do you want to keep these display settings?");
         let body = this._formatCountDown();
 
-        this._content = new Dialog.MessageDialogContent({ icon, title, body });
+        this._content = new Dialog.MessageDialogContent({
+            icon, title, body,
+            x_expand: true,
+            y_expand: true,
+        });
 
-        this.contentLayout.add(this._content,
-                               { x_fill: true,
-                                 y_fill: true });
+        this.contentLayout.add_child(this._content);
 
         /* Translators: this and the following message should be limited in length,
            to avoid ellipsizing the labels.
         */
         this._cancelButton = this.addButton({ label: _("Revert Settings"),
                                               action: this._onFailure.bind(this),
-                                              key: Clutter.Escape });
+                                              key: Clutter.KEY_Escape });
         this._okButton = this.addButton({ label: _("Keep Changes"),
                                           action: this._onSuccess.bind(this),
                                           default: true });
@@ -114,21 +119,20 @@ class DisplayChangeDialog extends ModalDialog.ModalDialog {
     }
 });
 
-var WindowDimmer = class {
-    constructor(actor) {
-        this._brightnessEffect = new Clutter.BrightnessContrastEffect({
-            name: 'dim',
-            enabled: false
+var WindowDimmer = GObject.registerClass(
+class WindowDimmer extends Clutter.BrightnessContrastEffect {
+    _init() {
+        super._init({
+            name: WINDOW_DIMMER_EFFECT_NAME,
+            enabled: false,
         });
-        actor.add_effect(this._brightnessEffect);
-        this.actor = actor;
         this._enabled = true;
     }
 
     _syncEnabled() {
-        let animating = this.actor.get_transition('@effects.dim.brightness') != null;
-        let dimmed = this._brightnessEffect.brightness.red != 127;
-        this._brightnessEffect.enabled = this._enabled && (animating || dimmed);
+        let animating = this.actor.get_transition(`@effects.${this.name}.brightness`) != null;
+        let dimmed = this.brightness.red != 127;
+        this.enabled = this._enabled && (animating || dimmed);
     }
 
     setEnabled(enabled) {
@@ -140,28 +144,27 @@ var WindowDimmer = class {
         let val = 127 * (1 + (dimmed ? 1 : 0) * DIM_BRIGHTNESS);
         let color = Clutter.Color.new(val, val, val, 255);
 
-        this.actor.ease_property('@effects.dim.brightness', color, {
+        this.actor.ease_property(`@effects.${this.name}.brightness`, color, {
             mode: Clutter.AnimationMode.LINEAR,
             duration: (dimmed ? DIM_TIME : UNDIM_TIME) * (animate ? 1 : 0),
-            onComplete: () => this._syncEnabled()
+            onComplete: () => this._syncEnabled(),
         });
 
         this._syncEnabled();
     }
-};
+});
 
 function getWindowDimmer(actor) {
     let enabled = Meta.prefs_get_attach_modal_dialogs();
-    if (actor._windowDimmer)
-        actor._windowDimmer.setEnabled(enabled);
+    let effect = actor.get_effect(WINDOW_DIMMER_EFFECT_NAME);
 
-    if (enabled) {
-        if (!actor._windowDimmer)
-            actor._windowDimmer = new WindowDimmer(actor);
-        return actor._windowDimmer;
-    } else {
-        return null;
+    if (effect) {
+        effect.setEnabled(enabled);
+    } else if (enabled) {
+        effect = new WindowDimmer();
+        actor.add_effect(effect);
     }
+    return effect;
 }
 
 /*
@@ -379,27 +382,28 @@ var WorkspaceTracker = class {
     }
 };
 
-var TilePreview = class {
-    constructor() {
-        this.actor = new St.Widget();
-        global.window_group.add_actor(this.actor);
+var TilePreview = GObject.registerClass(
+class TilePreview extends St.Widget {
+    _init() {
+        super._init();
+        global.window_group.add_actor(this);
 
         this._reset();
         this._showing = false;
     }
 
-    show(window, tileRect, monitorIndex) {
+    open(window, tileRect, monitorIndex) {
         let windowActor = window.get_compositor_private();
         if (!windowActor)
             return;
 
-        global.window_group.set_child_below_sibling(this.actor, windowActor);
+        global.window_group.set_child_below_sibling(this, windowActor);
 
         if (this._rect && this._rect.equal(tileRect))
             return;
 
-        let changeMonitor = (this._monitorIndex == -1 ||
-                             this._monitorIndex != monitorIndex);
+        let changeMonitor = this._monitorIndex == -1 ||
+                             this._monitorIndex != monitorIndex;
 
         this._monitorIndex = monitorIndex;
         this._rect = tileRect;
@@ -414,39 +418,39 @@ var TilePreview = class {
                                                    width: monitor.width,
                                                    height: monitor.height });
             let [, rect] = window.get_frame_rect().intersect(monitorRect);
-            this.actor.set_size(rect.width, rect.height);
-            this.actor.set_position(rect.x, rect.y);
-            this.actor.opacity = 0;
+            this.set_size(rect.width, rect.height);
+            this.set_position(rect.x, rect.y);
+            this.opacity = 0;
         }
 
         this._showing = true;
-        this.actor.show();
-        this.actor.ease({
+        this.show();
+        this.ease({
             x: tileRect.x,
             y: tileRect.y,
             width: tileRect.width,
             height: tileRect.height,
             opacity: 255,
             duration: WINDOW_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
         });
     }
 
-    hide() {
+    close() {
         if (!this._showing)
             return;
 
         this._showing = false;
-        this.actor.ease({
+        this.ease({
             opacity: 0,
             duration: WINDOW_ANIMATION_TIME,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => this._reset()
+            onComplete: () => this._reset(),
         });
     }
 
     _reset() {
-        this.actor.hide();
+        this.hide();
         this._rect = null;
         this._monitorIndex = -1;
     }
@@ -460,9 +464,9 @@ var TilePreview = class {
         if (this._rect.x + this._rect.width == monitor.x + monitor.width)
             styles.push('tile-preview-right');
 
-        this.actor.style_class = styles.join(' ');
+        this.style_class = styles.join(' ');
     }
-};
+});
 
 var TouchpadWorkspaceSwitchAction = class {
     constructor(actor, allowedModes) {
@@ -490,13 +494,13 @@ var TouchpadWorkspaceSwitchAction = class {
     _checkActivated() {
         let dir;
 
-        if (this._dy < -MOTION_THRESHOLD)
+        if (this._dy < -WS_MOTION_THRESHOLD)
             dir = Meta.MotionDirection.DOWN;
-        else if (this._dy > MOTION_THRESHOLD)
+        else if (this._dy > WS_MOTION_THRESHOLD)
             dir = Meta.MotionDirection.UP;
-        else if (this._dx < -MOTION_THRESHOLD)
+        else if (this._dx < -WS_MOTION_THRESHOLD)
             dir = Meta.MotionDirection.RIGHT;
-        else if (this._dx > MOTION_THRESHOLD)
+        else if (this._dx > WS_MOTION_THRESHOLD)
             dir = Meta.MotionDirection.LEFT;
         else
             return false;
@@ -523,14 +527,14 @@ var TouchpadWorkspaceSwitchAction = class {
 
             // Scale deltas up a bit to make it feel snappier
             this._dx += dx * 2;
-            if (!(this._touchpadSettings.get_boolean('natural-scroll')))
+            if (!this._touchpadSettings.get_boolean('natural-scroll'))
                 this._dy -= dy * 2;
             else
                 this._dy += dy * 2;
 
             this.emit('motion', this._dx, this._dy);
         } else {
-            if ((event.get_gesture_phase() == Clutter.TouchpadGesturePhase.END && ! this._checkActivated()) ||
+            if ((event.get_gesture_phase() == Clutter.TouchpadGesturePhase.END && !this._checkActivated()) ||
                 event.get_gesture_phase() == Clutter.TouchpadGesturePhase.CANCEL)
                 this.emit('cancel');
 
@@ -565,7 +569,7 @@ var WorkspaceSwitchAction = GObject.registerClass({
         if (!super.vfunc_gesture_prepare(actor))
             return false;
 
-        return (this._allowedModes & Main.actionMode);
+        return this._allowedModes & Main.actionMode;
     }
 
     vfunc_gesture_progress(_actor) {
@@ -583,8 +587,8 @@ var WorkspaceSwitchAction = GObject.registerClass({
     vfunc_swipe(actor, direction) {
         let [x, y] = this.get_motion_coords(0);
         let [xPress, yPress] = this.get_press_coords(0);
-        if (Math.abs(x - xPress) < MOTION_THRESHOLD &&
-            Math.abs(y - yPress) < MOTION_THRESHOLD) {
+        if (Math.abs(x - xPress) < WS_MOTION_THRESHOLD &&
+            Math.abs(y - yPress) < WS_MOTION_THRESHOLD) {
             this.emit('cancel');
             return;
         }
@@ -631,7 +635,7 @@ var AppSwitchAction = GObject.registerClass({
         const LONG_PRESS_TIMEOUT = 250;
 
         let nPoints = this.get_n_current_points();
-        let event = this.get_last_event (nPoints - 1);
+        let event = this.get_last_event(nPoints - 1);
 
         if (nPoints == 3) {
             this._longPressStartTime = event.get_time();
@@ -651,15 +655,14 @@ var AppSwitchAction = GObject.registerClass({
     }
 
     vfunc_gesture_progress(_actor) {
-        const MOTION_THRESHOLD = 30;
 
         if (this.get_n_current_points() == 3) {
             for (let i = 0; i < this.get_n_current_points(); i++) {
                 let [startX, startY] = this.get_press_coords(i);
                 let [x, y] = this.get_motion_coords(i);
 
-                if (Math.abs(x - startX) > MOTION_THRESHOLD ||
-                    Math.abs(y - startY) > MOTION_THRESHOLD)
+                if (Math.abs(x - startX) > APP_MOTION_THRESHOLD ||
+                    Math.abs(y - startY) > APP_MOTION_THRESHOLD)
                     return false;
             }
 
@@ -669,15 +672,16 @@ var AppSwitchAction = GObject.registerClass({
     }
 });
 
-var ResizePopup = class {
-    constructor() {
-        this._widget = new St.Widget({ layout_manager: new Clutter.BinLayout() });
+var ResizePopup = GObject.registerClass(
+class ResizePopup extends St.Widget {
+    _init() {
+        super._init({ layout_manager: new Clutter.BinLayout() });
         this._label = new St.Label({ style_class: 'resize-popup',
                                      x_align: Clutter.ActorAlign.CENTER,
                                      y_align: Clutter.ActorAlign.CENTER,
                                      x_expand: true, y_expand: true });
-        this._widget.add_child(this._label);
-        Main.uiGroup.add_actor(this._widget);
+        this.add_child(this._label);
+        Main.uiGroup.add_actor(this);
     }
 
     set(rect, displayW, displayH) {
@@ -686,15 +690,10 @@ var ResizePopup = class {
         let text = _("%d × %d").format(displayW, displayH);
         this._label.set_text(text);
 
-        this._widget.set_position(rect.x, rect.y);
-        this._widget.set_size(rect.width, rect.height);
+        this.set_position(rect.x, rect.y);
+        this.set_size(rect.width, rect.height);
     }
-
-    destroy() {
-        this._widget.destroy();
-        this._widget = null;
-    }
-};
+});
 
 var WindowManager = class {
     constructor() {
@@ -1024,15 +1023,14 @@ var WindowManager = class {
         this._gsdWacomProxy = new GsdWacomProxy(Gio.DBus.session, GSD_WACOM_BUS_NAME,
                                                 GSD_WACOM_OBJECT_PATH,
                                                 (proxy, error) => {
-                                                    if (error) {
+                                                    if (error)
                                                         log(error.message);
-                                                    }
                                                 });
 
         global.display.connect('pad-mode-switch', (display, pad, group, mode) => {
             let labels = [];
 
-            //FIXME: Fix num buttons
+            // FIXME: Fix num buttons
             for (let i = 0; i < 50; i++) {
                 let str = display.get_pad_action_label(pad, Meta.PadActionType.BUTTON, i);
                 labels.push(str ? str : '');
@@ -1090,7 +1088,7 @@ var WindowManager = class {
         let mode = Shell.ActionMode.ALL & ~Shell.ActionMode.LOCK_SCREEN;
         let bottomDragAction = new EdgeDragAction.EdgeDragAction(St.Side.BOTTOM, mode);
         bottomDragAction.connect('activated', () => {
-            Main.keyboard.show(Main.layoutManager.bottomIndex);
+            Main.keyboard.open(Main.layoutManager.bottomIndex);
         });
         Main.layoutManager.connect('keyboard-visible-changed', (manager, visible) => {
             bottomDragAction.cancel();
@@ -1120,7 +1118,7 @@ var WindowManager = class {
         this._currentPadOsd = new PadOsd.PadOsd(device, settings, imagePath, editionMode, monitorIndex);
         this._currentPadOsd.connect('closed', () => (this._currentPadOsd = null));
 
-        return this._currentPadOsd.actor;
+        return this._currentPadOsd;
     }
 
     _switchWorkspaceMotion(action, xRel, yRel) {
@@ -1152,7 +1150,7 @@ var WindowManager = class {
             y: 0,
             duration: WINDOW_ANIMATION_TIME,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => this._finishWorkspaceSwitch(switchData)
+            onComplete: () => this._finishWorkspaceSwitch(switchData),
         });
     }
 
@@ -1171,9 +1169,8 @@ var WindowManager = class {
 
     _lookupIndex(windows, metaWindow) {
         for (let i = 0; i < windows.length; i++) {
-            if (windows[i].metaWindow == metaWindow) {
+            if (windows[i].metaWindow == metaWindow)
                 return i;
-            }
         }
         return -1;
     }
@@ -1183,8 +1180,8 @@ var WindowManager = class {
             let win = actor.metaWindow;
             let workspaceManager = global.workspace_manager;
             let activeWorkspace = workspaceManager.get_active_workspace();
-            return (!win.is_override_redirect() &&
-                    win.located_on_workspace(activeWorkspace));
+            return !win.is_override_redirect() &&
+                    win.located_on_workspace(activeWorkspace);
         });
 
         if (windows.length == 0)
@@ -1196,7 +1193,7 @@ var WindowManager = class {
         if (focusWindow == null) {
             nextWindow = windows[0].metaWindow;
         } else {
-            let index = this._lookupIndex (windows, focusWindow) + 1;
+            let index = this._lookupIndex(windows, focusWindow) + 1;
 
             if (index >= windows.length)
                 index = 0;
@@ -1323,7 +1320,7 @@ var WindowManager = class {
                         this._minimizeWindowDone(shellwm, actor);
                     else
                         this._minimizeWindowOverwritten(shellwm, actor);
-                }
+                },
             });
         } else {
             let xDest, yDest, xScale, yScale;
@@ -1354,12 +1351,7 @@ var WindowManager = class {
                 y: yDest,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_IN_EXPO,
-                onStopped: isFinished => {
-                    if (isFinished)
-                        this._minimizeWindowDone(shellwm, actor);
-                    else
-                        this._minimizeWindowOverwritten(shellwm, actor);
-                }
+                onStopped: () => this._minimizeWindowDone(shellwm, actor),
             });
         }
     }
@@ -1375,12 +1367,6 @@ var WindowManager = class {
         }
     }
 
-    _minimizeWindowOverwritten(shellwm, actor) {
-        if (this._minimizing.delete(actor)) {
-            shellwm.completed_minimize(actor);
-        }
-    }
-
     _unminimizeWindow(shellwm, actor) {
         let types = [Meta.WindowType.NORMAL,
                      Meta.WindowType.MODAL_DIALOG,
@@ -1399,12 +1385,7 @@ var WindowManager = class {
                 opacity: 255,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-                onStopped: isFinished => {
-                    if (isFinished)
-                        this._unminimizeWindowDone(shellwm, actor);
-                    else
-                        this._unminimizeWindowOverwritten(shellwm, actor);
-                }
+                onStopped: () => this._unminimizeWindowDone(shellwm, actor),
             });
         } else {
             let [success, geom] = actor.meta_window.get_icon_geometry();
@@ -1436,12 +1417,7 @@ var WindowManager = class {
                 y: yDest,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_IN_EXPO,
-                onStopped: isFinished => {
-                    if (isFinished)
-                        this._unminimizeWindowDone(shellwm, actor);
-                    else
-                        this._unminimizeWindowOverwritten(shellwm, actor);
-                }
+                onStopped: () => this._unminimizeWindowDone(shellwm, actor),
             });
         }
     }
@@ -1457,12 +1433,6 @@ var WindowManager = class {
         }
     }
 
-    _unminimizeWindowOverwritten(shellwm, actor) {
-        if (this._unminimizing.delete(actor)) {
-            shellwm.completed_unminimize(actor);
-        }
-    }
-
     _sizeChangeWindow(shellwm, actor, whichChange, oldFrameRect, _oldBufferRect) {
         let types = [Meta.WindowType.NORMAL];
         if (!this._shouldAnimateActor(actor, types)) {
@@ -1496,7 +1466,7 @@ var WindowManager = class {
         this._resizePending.add(actor);
         actor.__animationInfo = { clone: actorClone,
                                   oldRect: oldFrameRect,
-                                  destroyId: destroyId };
+                                  destroyId };
     }
 
     _sizeChangedWindow(shellwm, actor) {
@@ -1523,7 +1493,7 @@ var WindowManager = class {
             scale_y: scaleY,
             opacity: 0,
             duration: WINDOW_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
         });
 
         actor.translation_x = -targetRect.x + sourceRect.x;
@@ -1541,12 +1511,7 @@ var WindowManager = class {
             translation_y: 0,
             duration: WINDOW_ANIMATION_TIME,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onStopped: isFinished => {
-                if (isFinished)
-                    this._sizeChangeWindowDone(shellwm, actor);
-                else
-                    this._sizeChangeWindowOverwritten(shellwm, actor);
-            }
+            onStopped: () => this._sizeChangeWindowDone(shellwm, actor),
         });
 
         // Now unfreeze actor updates, to get it to the new size.
@@ -1579,11 +1544,6 @@ var WindowManager = class {
             this._shellwm.completed_size_change(actor);
     }
 
-    _sizeChangeWindowOverwritten(shellwm, actor) {
-        if (this._resizing.delete(actor))
-            this._clearAnimationInfo(actor);
-    }
-
     _hasAttachedDialogs(window, ignoreWindow) {
         var count = 0;
         window.foreach_transient(win => {
@@ -1681,12 +1641,7 @@ var WindowManager = class {
                 scale_y: 1,
                 duration: SHOW_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_EXPO,
-                onStopped: isFinished => {
-                    if (isFinished)
-                        this._mapWindowDone(shellwm, actor);
-                    else
-                        this._mapWindowOverwrite(shellwm, actor);
-                }
+                onStopped: () => this._mapWindowDone(shellwm, actor),
             });
             break;
         case Meta.WindowType.MODAL_DIALOG:
@@ -1703,12 +1658,7 @@ var WindowManager = class {
                 scale_y: 1,
                 duration: DIALOG_SHOW_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-                onStopped: isFinished => {
-                    if (isFinished)
-                        this._mapWindowDone(shellwm, actor);
-                    else
-                        this._mapWindowOverwrite(shellwm, actor);
-                }
+                onStopped: () => this._mapWindowDone(shellwm, actor),
             });
             break;
         default:
@@ -1729,12 +1679,6 @@ var WindowManager = class {
         }
     }
 
-    _mapWindowOverwrite(shellwm, actor) {
-        if (this._mapping.delete(actor)) {
-            shellwm.completed_map(actor);
-        }
-    }
-
     _destroyWindow(shellwm, actor) {
         let window = actor.meta_window;
         if (actor._notifyWindowTypeSignalId) {
@@ -1768,7 +1712,7 @@ var WindowManager = class {
                 scale_y: 0.8,
                 duration: DESTROY_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-                onStopped: () => this._destroyWindowDone(shellwm, actor)
+                onStopped: () => this._destroyWindowDone(shellwm, actor),
             });
             break;
         case Meta.WindowType.MODAL_DIALOG:
@@ -1788,7 +1732,7 @@ var WindowManager = class {
                 scale_y: 0,
                 duration: DIALOG_DESTROY_WINDOW_ANIMATION_TIME,
                 mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-                onStopped: () => this._destroyWindowDone(shellwm, actor)
+                onStopped: () => this._destroyWindowDone(shellwm, actor),
             });
             break;
         default:
@@ -1906,7 +1850,7 @@ var WindowManager = class {
         wgroup.add_actor(switchData.container);
 
         let workspaceManager = global.workspace_manager;
-        let curWs = workspaceManager.get_workspace_by_index (from);
+        let curWs = workspaceManager.get_workspace_by_index(from);
 
         for (let dir of Object.values(Meta.MotionDirection)) {
             let ws = null;
@@ -1925,13 +1869,13 @@ var WindowManager = class {
                          actor: new Clutter.Actor() };
             switchData.surroundings[dir] = info;
             switchData.container.add_actor(info.actor);
-            info.actor.raise_top();
+            switchData.container.set_child_above_sibling(info.actor, null);
 
             let [x, y] = this._getPositionForDirection(dir, curWs, ws);
             info.actor.set_position(x, y);
         }
 
-        switchData.movingWindowBin.raise_top();
+        wgroup.set_child_above_sibling(switchData.movingWindowBin, null);
 
         for (let i = 0; i < windows.length; i++) {
             let actor = windows[i];
@@ -1947,12 +1891,14 @@ var WindowManager = class {
                            parent: actor.get_parent() };
 
             if (this._movingWindow && window == this._movingWindow) {
+                record.parent.remove_child(actor);
                 switchData.movingWindow = record;
                 switchData.windows.push(switchData.movingWindow);
-                actor.reparent(switchData.movingWindowBin);
+                switchData.movingWindowBin.add_child(actor);
             } else if (window.get_workspace().index() == from) {
+                record.parent.remove_child(actor);
                 switchData.windows.push(record);
-                actor.reparent(switchData.curGroup);
+                switchData.curGroup.add_child(actor);
             } else {
                 let visible = false;
                 for (let dir of Object.values(Meta.MotionDirection)) {
@@ -1961,8 +1907,9 @@ var WindowManager = class {
                     if (!info || info.index != window.get_workspace().index())
                         continue;
 
+                    record.parent.remove_child(actor);
                     switchData.windows.push(record);
-                    actor.reparent(info.actor);
+                    info.actor.add_child(actor);
                     visible = true;
                     break;
                 }
@@ -1987,7 +1934,8 @@ var WindowManager = class {
             let w = switchData.windows[i];
 
             w.window.disconnect(w.windowDestroyId);
-            w.window.reparent(w.parent);
+            w.window.get_parent().remove_child(w.window);
+            w.parent.add_child(w.window);
 
             if (w.window.get_meta_window().get_workspace() !=
                 global.workspace_manager.get_active_workspace())
@@ -2030,7 +1978,7 @@ var WindowManager = class {
             y: yDest,
             duration: WINDOW_ANIMATION_TIME,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => this._switchWorkspaceDone(shellwm)
+            onComplete: () => this._switchWorkspaceDone(shellwm),
         });
     }
 
@@ -2042,13 +1990,13 @@ var WindowManager = class {
     _showTilePreview(shellwm, window, tileRect, monitorIndex) {
         if (!this._tilePreview)
             this._tilePreview = new TilePreview();
-        this._tilePreview.show(window, tileRect, monitorIndex);
+        this._tilePreview.open(window, tileRect, monitorIndex);
     }
 
     _hideTilePreview() {
         if (!this._tilePreview)
             return;
-        this._tilePreview.hide();
+        this._tilePreview.close();
     }
 
     _showWindowMenu(shellwm, window, menu, rect) {
@@ -2176,7 +2124,7 @@ var WindowManager = class {
                 else
                     direction = Meta.MotionDirection.LEFT;
             } else {
-                if (vertical)
+                if (vertical) // eslint-disable-line no-lonely-if
                     direction = Meta.MotionDirection.DOWN;
                 else if (rtl)
                     direction = Meta.MotionDirection.LEFT;
@@ -2240,7 +2188,7 @@ var WindowManager = class {
             window.change_workspace(workspace);
 
             global.display.clear_mouse_mode();
-            workspace.activate_with_focus (window, global.get_current_time());
+            workspace.activate_with_focus(window, global.get_current_time());
         }
     }
 
@@ -2264,10 +2212,11 @@ var WindowManager = class {
 
             this._resizePopup.set(rect, displayW, displayH);
         } else {
-            if (this._resizePopup) {
-                this._resizePopup.destroy();
-                this._resizePopup = null;
-            }
+            if (!this._resizePopup)
+                return;
+
+            this._resizePopup.destroy();
+            this._resizePopup = null;
         }
     }
 };
