@@ -29,10 +29,8 @@
 #include <meta/meta-workspace-manager.h>
 #include <meta/meta-x11-display.h>
 
-#ifdef HAVE_GNOME_SYSTEMD
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-systemd.h>
-#endif
 
 /* Memory report bits */
 #ifdef HAVE_MALLINFO
@@ -1314,7 +1312,6 @@ shell_global_get_current_time (ShellGlobal *global)
   return clutter_get_current_event_time ();
 }
 
-#ifdef HAVE_GNOME_SYSTEMD
 static void
 shell_global_app_launched_cb (GAppLaunchContext *context,
                               GAppInfo          *info,
@@ -1338,7 +1335,6 @@ shell_global_app_launched_cb (GAppLaunchContext *context,
                              NULL,
                              NULL, NULL, NULL);
 }
-#endif
 
 /**
  * shell_global_create_app_launch_context:
@@ -1375,12 +1371,10 @@ shell_global_create_app_launch_context (ShellGlobal *global,
 
   meta_launch_context_set_workspace (context, ws);
 
-#ifdef HAVE_GNOME_SYSTEMD
   g_signal_connect (context,
                     "launched",
                     G_CALLBACK (shell_global_app_launched_cb),
                     NULL);
-#endif
 
   return (GAppLaunchContext *) context;
 }
@@ -1554,55 +1548,6 @@ delete_variant_cb (GObject      *object,
   g_hash_table_remove (global->save_ops, object);
 }
 
-static void
-replace_contents_worker (GTask        *task,
-                         gpointer      source_object,
-                         gpointer      task_data,
-                         GCancellable *cancellable)
-{
-  GFile *file = source_object;
-  GBytes *bytes = task_data;
-  GError *error = NULL;
-  const gchar *data;
-  gsize len;
-
-  data = g_bytes_get_data (bytes, &len);
-
-  if (!g_file_replace_contents (file, data, len, NULL, FALSE,
-                                G_FILE_CREATE_REPLACE_DESTINATION,
-                                NULL, cancellable, &error))
-    g_task_return_error (task, g_steal_pointer (&error));
-  else
-    g_task_return_boolean (task, TRUE);
-}
-
-static void
-replace_contents_async (GFile               *path,
-                        GBytes              *bytes,
-                        GCancellable        *cancellable,
-                        GAsyncReadyCallback  callback,
-                        gpointer             user_data)
-{
-  g_autoptr(GTask) task = NULL;
-
-  g_assert (G_IS_FILE (path));
-  g_assert (bytes != NULL);
-  g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
-
-  task = g_task_new (path, cancellable, callback, user_data);
-  g_task_set_source_tag (task, replace_contents_async);
-  g_task_set_task_data (task, g_bytes_ref (bytes), (GDestroyNotify)g_bytes_unref);
-  g_task_run_in_thread (task, replace_contents_worker);
-}
-
-static gboolean
-replace_contents_finish (GFile         *file,
-                         GAsyncResult  *result,
-                         GError       **error)
-{
-  return g_task_propagate_boolean (G_TASK (result), error);
-}
-
 static void
 replace_variant_cb (GObject      *object,
                     GAsyncResult *result,
@@ -1611,7 +1556,7 @@ replace_variant_cb (GObject      *object,
   ShellGlobal *global = user_data;
   GError *error = NULL;
 
-  if (!replace_contents_finish (G_FILE (object), result, &error))
+  if (!g_file_replace_contents_finish (G_FILE (object), result, NULL, &error))
     {
       if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         {
@@ -1647,19 +1592,12 @@ save_variant (ShellGlobal *global,
     }
   else
     {
-      g_autoptr(GBytes) bytes = NULL;
-
-      bytes = g_bytes_new_with_free_func (g_variant_get_data (variant),
-                                          g_variant_get_size (variant),
-                                          (GDestroyNotify)g_variant_unref,
-                                          g_variant_ref (variant));
-      /* g_file_replace_contents_async() can potentially fsync() from the
-       * calling thread when completing the asynchronous task. Instead, we
-       * want to force that fsync() to a thread to avoid blocking the
-       * compositor main loop. Using our own replace_contents_async()
-       * simply executes the operation synchronously from a thread.
-       */
-      replace_contents_async (path, bytes, cancellable, replace_variant_cb, global);
+      g_file_replace_contents_async (path,
+                                     g_variant_get_data (variant),
+                                     g_variant_get_size (variant),
+                                     NULL, FALSE,
+                                     G_FILE_CREATE_REPLACE_DESTINATION,
+                                     cancellable, replace_variant_cb, global);
     }
 
   g_object_unref (path);
