@@ -56,6 +56,7 @@
 
 #include "st-icon.h"
 #include "st-label.h"
+#include "st-settings.h"
 #include "st-widget.h"
 #include "st-texture-cache.h"
 #include "st-clipboard.h"
@@ -79,8 +80,12 @@ enum
   PROP_TEXT,
   PROP_INPUT_PURPOSE,
   PROP_INPUT_HINTS,
+
+  N_PROPS
 };
 
+static GParamSpec *props[N_PROPS] = { NULL, };
+
 /* signals */
 enum
 {
@@ -108,7 +113,7 @@ struct _StEntryPrivate
   gboolean      capslock_warning_shown;
   gboolean      has_ibeam;
 
-  CoglHandle    text_shadow_material;
+  CoglPipeline *text_shadow_material;
   gfloat        shadow_width;
   gfloat        shadow_height;
 };
@@ -239,15 +244,15 @@ remove_capslock_feedback (StEntry *entry)
 }
 
 static void
-keymap_state_changed (GdkKeymap *keymap,
-                      gpointer   user_data)
+keymap_state_changed (ClutterKeymap *keymap,
+                      gpointer       user_data)
 {
   StEntry *entry = ST_ENTRY (user_data);
   StEntryPrivate *priv = ST_ENTRY_PRIV (entry);
 
   if (clutter_text_get_password_char (CLUTTER_TEXT (priv->entry)) != 0)
     {
-      if (gdk_keymap_get_caps_lock_state (keymap))
+      if (clutter_keymap_get_caps_lock_state (keymap))
         show_capslock_feedback (entry);
       else
         remove_capslock_feedback (entry);
@@ -259,15 +264,11 @@ st_entry_dispose (GObject *object)
 {
   StEntry *entry = ST_ENTRY (object);
   StEntryPrivate *priv = ST_ENTRY_PRIV (entry);
-  GdkKeymap *keymap;
+  ClutterKeymap *keymap;
 
-  if (priv->text_shadow_material != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->text_shadow_material);
-      priv->text_shadow_material = COGL_INVALID_HANDLE;
-    }
+  cogl_clear_object (&priv->text_shadow_material);
 
-  keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+  keymap = clutter_backend_get_keymap (clutter_get_default_backend ());
   g_signal_handlers_disconnect_by_func (keymap, keymap_state_changed, entry);
 
   G_OBJECT_CLASS (st_entry_parent_class)->dispose (object);
@@ -301,11 +302,7 @@ st_entry_style_changed (StWidget *self)
   gchar *font_string, *font_name;
   gdouble size;
 
-  if (priv->text_shadow_material != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->text_shadow_material);
-      priv->text_shadow_material = COGL_INVALID_HANDLE;
-    }
+  cogl_clear_object (&priv->text_shadow_material);
 
   theme_node = st_widget_get_theme_node (self);
 
@@ -338,7 +335,7 @@ st_entry_style_changed (StWidget *self)
 static gboolean
 st_entry_navigate_focus (StWidget         *widget,
                          ClutterActor     *from,
-                         GtkDirectionType  direction)
+                         StDirectionType   direction)
 {
   StEntryPrivate *priv = ST_ENTRY_PRIV (widget);
 
@@ -571,11 +568,11 @@ clutter_text_focus_in_cb (ClutterText  *text,
                           ClutterActor *actor)
 {
   StEntry *entry = ST_ENTRY (actor);
-  GdkKeymap *keymap;
+  ClutterKeymap *keymap;
 
   st_entry_update_hint_visibility (entry);
 
-  keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+  keymap = clutter_backend_get_keymap (clutter_get_default_backend ());
   keymap_state_changed (keymap, entry);
   g_signal_connect (keymap, "state-changed",
                     G_CALLBACK (keymap_state_changed), entry);
@@ -589,7 +586,7 @@ clutter_text_focus_out_cb (ClutterText  *text,
                            ClutterActor *actor)
 {
   StEntry *entry = ST_ENTRY (actor);
-  GdkKeymap *keymap;
+  ClutterKeymap *keymap;
 
   st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "focus");
 
@@ -598,7 +595,7 @@ clutter_text_focus_out_cb (ClutterText  *text,
   clutter_text_set_cursor_visible (text, FALSE);
   remove_capslock_feedback (entry);
 
-  keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+  keymap = clutter_backend_get_keymap (clutter_get_default_backend ());
   g_signal_handlers_disconnect_by_func (keymap, keymap_state_changed, entry);
 }
 
@@ -623,11 +620,7 @@ clutter_text_changed_cb (GObject    *object,
   StEntryPrivate *priv = ST_ENTRY_PRIV (entry);
 
   /* Since the text changed, force a regen of the shadow texture */
-  if (priv->text_shadow_material != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->text_shadow_material);
-      priv->text_shadow_material = COGL_INVALID_HANDLE;
-    }
+  cogl_clear_object (&priv->text_shadow_material);
 }
 
 static void
@@ -656,29 +649,32 @@ clutter_text_button_press_event (ClutterActor       *actor,
                                  gpointer            user_data)
 {
   StEntryPrivate *priv = ST_ENTRY_PRIV (user_data);
-  GtkSettings *settings = gtk_settings_get_default ();
-  gboolean primary_paste_enabled;
 
-  g_object_get (settings,
-                "gtk-enable-primary-paste", &primary_paste_enabled,
-                NULL);
-
-  if (primary_paste_enabled && event->button == 2
-      && clutter_text_get_editable (CLUTTER_TEXT (priv->entry)))
+  if (event->button == 2 &&
+      clutter_text_get_editable (CLUTTER_TEXT (priv->entry)))
     {
-      StClipboard *clipboard;
+      StSettings *settings;
+      gboolean primary_paste_enabled;
 
-      clipboard = st_clipboard_get_default ();
+      settings = st_settings_get ();
+      g_object_get (settings, "primary-paste", &primary_paste_enabled, NULL);
 
-      /* By the time the clipboard callback is called,
-       * the rest of the signal handlers will have
-       * run, making the text cursor to be in the correct
-       * place.
-       */
-      st_clipboard_get_text (clipboard,
-                             ST_CLIPBOARD_TYPE_PRIMARY,
-                             st_entry_clipboard_callback,
-                             user_data);
+      if (primary_paste_enabled)
+        {
+          StClipboard *clipboard;
+
+          clipboard = st_clipboard_get_default ();
+
+          /* By the time the clipboard callback is called,
+           * the rest of the signal handlers will have
+           * run, making the text cursor to be in the correct
+           * place.
+           */
+          st_clipboard_get_text (clipboard,
+                                 ST_CLIPBOARD_TYPE_PRIMARY,
+                                 st_entry_clipboard_callback,
+                                 user_data);
+        }
     }
 
   return FALSE;
@@ -846,14 +842,15 @@ st_entry_leave_event (ClutterActor         *actor,
 }
 
 static void
-st_entry_paint (ClutterActor *actor)
+st_entry_paint (ClutterActor        *actor,
+                ClutterPaintContext *paint_context)
 {
   StEntryPrivate *priv = ST_ENTRY_PRIV (actor);
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
   StShadow *shadow_spec = st_theme_node_get_text_shadow (theme_node);
   ClutterActorClass *parent_class;
 
-  st_widget_paint_background (ST_WIDGET (actor));
+  st_widget_paint_background (ST_WIDGET (actor), paint_context);
 
   if (shadow_spec)
     {
@@ -863,14 +860,13 @@ st_entry_paint (ClutterActor *actor)
       clutter_actor_get_allocation_box (priv->entry, &allocation);
       clutter_actor_box_get_size (&allocation, &width, &height);
 
-      if (priv->text_shadow_material == COGL_INVALID_HANDLE ||
+      if (priv->text_shadow_material == NULL ||
           width != priv->shadow_width ||
           height != priv->shadow_height)
         {
-          CoglHandle material;
+          CoglPipeline *material;
 
-          if (priv->text_shadow_material != COGL_INVALID_HANDLE)
-            cogl_handle_unref (priv->text_shadow_material);
+          cogl_clear_object (&priv->text_shadow_material);
 
           material = _st_create_shadow_pipeline_from_actor (shadow_spec,
                                                             priv->entry);
@@ -880,11 +876,17 @@ st_entry_paint (ClutterActor *actor)
           priv->text_shadow_material = material;
         }
 
-      if (priv->text_shadow_material != COGL_INVALID_HANDLE)
-        _st_paint_shadow_with_opacity (shadow_spec,
-                                       priv->text_shadow_material,
-                                       &allocation,
-                                       clutter_actor_get_paint_opacity (priv->entry));
+      if (priv->text_shadow_material != NULL)
+        {
+          CoglFramebuffer *framebuffer =
+            clutter_paint_context_get_framebuffer (paint_context);
+
+          _st_paint_shadow_with_opacity (shadow_spec,
+                                         framebuffer,
+                                         priv->text_shadow_material,
+                                         &allocation,
+                                         clutter_actor_get_paint_opacity (priv->entry));
+        }
     }
 
   /* Since we paint the background ourselves, chain to the parent class
@@ -892,7 +894,7 @@ st_entry_paint (ClutterActor *actor)
    * This is needed as we still want to paint children.
    */
   parent_class = g_type_class_peek_parent (st_entry_parent_class);
-  parent_class->paint (actor);
+  parent_class->paint (actor, paint_context);
 }
 
 static void
@@ -918,7 +920,6 @@ st_entry_class_init (StEntryClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
-  GParamSpec *pspec;
 
   gobject_class->set_property = st_entry_set_property;
   gobject_class->get_property = st_entry_get_property;
@@ -941,66 +942,67 @@ st_entry_class_init (StEntryClass *klass)
   widget_class->navigate_focus = st_entry_navigate_focus;
   widget_class->get_accessible_type = st_entry_accessible_get_type;
 
-  pspec = g_param_spec_object ("clutter-text",
-			       "Clutter Text",
-			       "Internal ClutterText actor",
-			       CLUTTER_TYPE_TEXT,
-			       G_PARAM_READABLE);
-  g_object_class_install_property (gobject_class, PROP_CLUTTER_TEXT, pspec);
+  props[PROP_CLUTTER_TEXT] =
+    g_param_spec_object ("clutter-text",
+                         "Clutter Text",
+                         "Internal ClutterText actor",
+                         CLUTTER_TYPE_TEXT,
+                         ST_PARAM_READABLE);
 
-  pspec = g_param_spec_object ("primary-icon",
-			       "Primary Icon",
-			       "Primary Icon actor",
-			       CLUTTER_TYPE_ACTOR,
-			       G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_PRIMARY_ICON, pspec);
+  props[PROP_PRIMARY_ICON] =
+    g_param_spec_object ("primary-icon",
+                         "Primary Icon",
+                         "Primary Icon actor",
+                         CLUTTER_TYPE_ACTOR,
+                         ST_PARAM_READWRITE);
 
-  pspec = g_param_spec_object ("secondary-icon",
-			       "Secondary Icon",
-			       "Secondary Icon actor",
-			       CLUTTER_TYPE_ACTOR,
-			       G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_SECONDARY_ICON, pspec);
+  props[PROP_SECONDARY_ICON] =
+    g_param_spec_object ("secondary-icon",
+                         "Secondary Icon",
+                         "Secondary Icon actor",
+                         CLUTTER_TYPE_ACTOR,
+                         ST_PARAM_READWRITE);
 
-  pspec = g_param_spec_string ("hint-text",
-                               "Hint Text",
-                               "Text to display when the entry is not focused "
-                               "and the text property is empty",
-                               NULL, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_HINT_TEXT, pspec);
+  props[PROP_HINT_TEXT] =
+    g_param_spec_string ("hint-text",
+                         "Hint Text",
+                         "Text to display when the entry is not focused "
+                         "and the text property is empty",
+                         NULL,
+                         ST_PARAM_READWRITE);
 
-  pspec = g_param_spec_object ("hint-actor",
-                               "Hint Actor",
-                               "An actor to display when the entry is not focused "
-                               "and the text property is empty",
-                               CLUTTER_TYPE_ACTOR,
-                               G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_HINT_ACTOR, pspec);
+  props[PROP_HINT_ACTOR] =
+    g_param_spec_object ("hint-actor",
+                         "Hint Actor",
+                         "An actor to display when the entry is not focused "
+                         "and the text property is empty",
+                         CLUTTER_TYPE_ACTOR,
+                         ST_PARAM_READWRITE);
 
-  pspec = g_param_spec_string ("text",
-                               "Text",
-                               "Text of the entry",
-                               NULL, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_TEXT, pspec);
+  props[PROP_TEXT] =
+    g_param_spec_string ("text",
+                         "Text",
+                         "Text of the entry",
+                         NULL,
+                         ST_PARAM_READWRITE);
 
-  pspec = g_param_spec_enum ("input-purpose",
-                             "Purpose",
-                             "Purpose of the text field",
-                             CLUTTER_TYPE_INPUT_CONTENT_PURPOSE,
-                             CLUTTER_INPUT_CONTENT_PURPOSE_NORMAL,
-                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (gobject_class,
-                                   PROP_INPUT_PURPOSE,
-                                   pspec);
+  props[PROP_INPUT_PURPOSE] =
+    g_param_spec_enum ("input-purpose",
+                       "Purpose",
+                       "Purpose of the text field",
+                       CLUTTER_TYPE_INPUT_CONTENT_PURPOSE,
+                       CLUTTER_INPUT_CONTENT_PURPOSE_NORMAL,
+                       ST_PARAM_READWRITE);
 
-  pspec = g_param_spec_flags ("input-hints",
-                              "hints",
-                              "Hints for the text field behaviour",
-                              CLUTTER_TYPE_INPUT_CONTENT_HINT_FLAGS,
-                              0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (gobject_class,
-                                   PROP_INPUT_HINTS,
-                                   pspec);
+  props[PROP_INPUT_HINTS] =
+    g_param_spec_flags ("input-hints",
+                        "hints",
+                        "Hints for the text field behaviour",
+                        CLUTTER_TYPE_INPUT_CONTENT_HINT_FLAGS,
+                        0,
+                        ST_PARAM_READWRITE);
+
+  g_object_class_install_properties (gobject_class, N_PROPS, props);
 
   /* signals */
   /**
@@ -1063,7 +1065,7 @@ st_entry_init (StEntry *entry)
 
   priv->spacing = 6.0f;
 
-  priv->text_shadow_material = COGL_INVALID_HANDLE;
+  priv->text_shadow_material = NULL;
   priv->shadow_width = -1.;
   priv->shadow_height = -1.;
 
@@ -1138,7 +1140,7 @@ st_entry_set_text (StEntry     *entry,
 
   st_entry_update_hint_visibility (entry);
 
-  g_object_notify (G_OBJECT (entry), "text");
+  g_object_notify_by_pspec (G_OBJECT (entry), props[PROP_TEXT]);
 }
 
 /**
@@ -1228,7 +1230,7 @@ st_entry_set_input_purpose (StEntry                    *entry,
     {
       clutter_text_set_input_purpose (editable, purpose);
 
-      g_object_notify (G_OBJECT (entry), "input-purpose");
+      g_object_notify_by_pspec (G_OBJECT (entry), props[PROP_INPUT_PURPOSE]);
     }
 }
 
@@ -1273,7 +1275,7 @@ st_entry_set_input_hints (StEntry                      *entry,
     {
       clutter_text_set_input_hints (editable, hints);
 
-      g_object_notify (G_OBJECT (entry), "input-hints");
+      g_object_notify_by_pspec (G_OBJECT (entry), props[PROP_INPUT_HINTS]);
     }
 }
 
