@@ -57,7 +57,6 @@
 typedef struct _StWidgetPrivate        StWidgetPrivate;
 struct _StWidgetPrivate
 {
-  StTheme      *theme;
   StThemeNode  *theme_node;
   gchar        *pseudo_class;
   gchar        *style_class;
@@ -106,7 +105,6 @@ enum
 {
   PROP_0,
 
-  PROP_THEME,
   PROP_PSEUDO_CLASS,
   PROP_STYLE_CLASS,
   PROP_STYLE,
@@ -115,9 +113,13 @@ enum
   PROP_CAN_FOCUS,
   PROP_LABEL_ACTOR,
   PROP_ACCESSIBLE_ROLE,
-  PROP_ACCESSIBLE_NAME
+  PROP_ACCESSIBLE_NAME,
+
+  N_PROPS
 };
 
+static GParamSpec *props[N_PROPS] = { NULL, };
+
 enum
 {
   STYLE_CHANGED,
@@ -129,8 +131,6 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0, };
 
-gfloat st_slow_down_factor = 1.0;
-
 G_DEFINE_TYPE_WITH_PRIVATE (StWidget, st_widget, CLUTTER_TYPE_ACTOR);
 #define ST_WIDGET_PRIVATE(w) ((StWidgetPrivate *)st_widget_get_instance_private (w))
 
@@ -152,10 +152,6 @@ st_widget_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_THEME:
-      st_widget_set_theme (actor, g_value_get_object (value));
-      break;
-
     case PROP_PSEUDO_CLASS:
       st_widget_set_style_pseudo_class (actor, g_value_get_string (value));
       break;
@@ -209,10 +205,6 @@ st_widget_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_THEME:
-      g_value_set_object (value, priv->theme);
-      break;
-
     case PROP_PSEUDO_CLASS:
       g_value_set_string (value, priv->pseudo_class);
       break;
@@ -310,18 +302,14 @@ st_widget_dispose (GObject *gobject)
   StWidget *actor = ST_WIDGET (gobject);
   StWidgetPrivate *priv = st_widget_get_instance_private (actor);
 
-  g_clear_pointer (&priv->theme, g_object_unref);
   g_clear_pointer (&priv->theme_node, g_object_unref);
 
   st_widget_remove_transition (actor);
 
   g_clear_pointer (&priv->label_actor, g_object_unref);
 
-  if (priv->texture_file_changed_id != 0)
-    {
-      g_signal_handler_disconnect (st_texture_cache_get_default (), priv->texture_file_changed_id);
-      priv->texture_file_changed_id = 0;
-    }
+  g_clear_signal_handler (&priv->texture_file_changed_id,
+                          st_texture_cache_get_default ());
 
   g_clear_object (&priv->first_visible_child);
   g_clear_object (&priv->last_visible_child);
@@ -415,7 +403,8 @@ st_widget_allocate (ClutterActor          *actor,
  * painting children.
  */
 void
-st_widget_paint_background (StWidget *widget)
+st_widget_paint_background (StWidget            *widget,
+                            ClutterPaintContext *paint_context)
 {
   StWidgetPrivate *priv = st_widget_get_instance_private (widget);
   CoglFramebuffer *framebuffer;
@@ -427,7 +416,7 @@ st_widget_paint_background (StWidget *widget)
   if (!st_widget_get_resource_scale (widget, &resource_scale))
     return;
 
-  framebuffer = cogl_get_draw_framebuffer ();
+  framebuffer = clutter_paint_context_get_framebuffer (paint_context);
   theme_node = st_widget_get_theme_node (widget);
 
   clutter_actor_get_allocation_box (CLUTTER_ACTOR (widget), &allocation);
@@ -450,12 +439,13 @@ st_widget_paint_background (StWidget *widget)
 }
 
 static void
-st_widget_paint (ClutterActor *actor)
+st_widget_paint (ClutterActor        *actor,
+                 ClutterPaintContext *paint_context)
 {
-  st_widget_paint_background (ST_WIDGET (actor));
+  st_widget_paint_background (ST_WIDGET (actor), paint_context);
 
   /* Chain up so we paint children. */
-  CLUTTER_ACTOR_CLASS (st_widget_parent_class)->paint (actor);
+  CLUTTER_ACTOR_CLASS (st_widget_parent_class)->paint (actor, paint_context);
 }
 
 static void
@@ -637,7 +627,7 @@ st_widget_get_theme_node (StWidget *widget)
         pseudo_class = direction_pseudo_class;
 
       context = st_theme_context_get_for_stage (stage);
-      tmp_node = st_theme_node_new (context, parent_node, priv->theme,
+      tmp_node = st_theme_node_new (context, parent_node, NULL,
                                     G_OBJECT_TYPE (widget),
                                     clutter_actor_get_name (CLUTTER_ACTOR (widget)),
                                     priv->style_class,
@@ -760,7 +750,7 @@ st_widget_get_paint_volume (ClutterActor *self,
   ClutterActorBox paint_box, alloc_box;
   StThemeNode *theme_node;
   StWidgetPrivate *priv;
-  ClutterVertex origin;
+  graphene_point3d_t origin;
 
   /* Setting the paint volume does not make sense when we don't have any allocation */
   if (!clutter_actor_has_allocation (self))
@@ -834,7 +824,6 @@ st_widget_class_init (StWidgetClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  GParamSpec *pspec;
 
   gobject_class->set_property = st_widget_set_property;
   gobject_class->get_property = st_widget_get_property;
@@ -869,25 +858,24 @@ st_widget_class_init (StWidgetClass *klass)
    * The pseudo-class of the actor. Typical values include "hover", "active",
    * "focus".
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_PSEUDO_CLASS,
-                                   g_param_spec_string ("pseudo-class",
-                                                        "Pseudo Class",
-                                                        "Pseudo class for styling",
-                                                        "",
-                                                        ST_PARAM_READWRITE));
+  props[PROP_PSEUDO_CLASS] =
+    g_param_spec_string ("pseudo-class",
+                         "Pseudo Class",
+                         "Pseudo class for styling",
+                         "",
+                         ST_PARAM_READWRITE);
+
   /**
    * StWidget:style-class:
    *
    * The style-class of the actor for use in styling.
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_STYLE_CLASS,
-                                   g_param_spec_string ("style-class",
-                                                        "Style Class",
-                                                        "Style class for styling",
-                                                        "",
-                                                        ST_PARAM_READWRITE));
+  props[PROP_STYLE_CLASS] =
+    g_param_spec_string ("style-class",
+                         "Style Class",
+                         "Style class for styling",
+                         "",
+                         ST_PARAM_READWRITE);
 
   /**
    * StWidget:style:
@@ -895,27 +883,12 @@ st_widget_class_init (StWidgetClass *klass)
    * Inline style information for the actor as a ';'-separated list of
    * CSS properties.
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_STYLE,
-                                   g_param_spec_string ("style",
-                                                        "Style",
-                                                        "Inline style string",
-                                                        "",
-                                                        ST_PARAM_READWRITE));
-
-  /**
-   * StWidget:theme:
-   *
-   * A theme set on this actor overriding the global theming for this actor
-   * and its descendants
-   */
-  g_object_class_install_property (gobject_class,
-                                   PROP_THEME,
-                                   g_param_spec_object ("theme",
-                                                        "Theme",
-                                                        "Theme override",
-                                                        ST_TYPE_THEME,
-                                                        ST_PARAM_READWRITE));
+  props[PROP_STYLE] =
+     g_param_spec_string ("style",
+                          "Style",
+                          "Inline style string",
+                          "",
+                          ST_PARAM_READWRITE);
 
   /**
    * StWidget:track-hover:
@@ -926,14 +899,12 @@ st_widget_class_init (StWidgetClass *klass)
    * adjusted automatically as the pointer moves in and out of the
    * widget.
    */
-  pspec = g_param_spec_boolean ("track-hover",
-                                "Track hover",
-                                "Determines whether the widget tracks hover state",
-                                FALSE,
-                                ST_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_TRACK_HOVER,
-                                   pspec);
+  props[PROP_TRACK_HOVER] =
+     g_param_spec_boolean ("track-hover",
+                           "Track hover",
+                           "Determines whether the widget tracks hover state",
+                           FALSE,
+                           ST_PARAM_READWRITE);
 
   /**
    * StWidget:hover:
@@ -942,68 +913,63 @@ st_widget_class_init (StWidgetClass *klass)
    * only tracked automatically if #StWidget:track-hover is %TRUE, but you can
    * adjust it manually in any case.
    */
-  pspec = g_param_spec_boolean ("hover",
-                                "Hover",
-                                "Whether the pointer is hovering over the widget",
-                                FALSE,
-                                ST_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_HOVER,
-                                   pspec);
+  props[PROP_HOVER] =
+     g_param_spec_boolean ("hover",
+                           "Hover",
+                           "Whether the pointer is hovering over the widget",
+                           FALSE,
+                           ST_PARAM_READWRITE);
 
   /**
    * StWidget:can-focus:
    *
    * Whether or not the widget can be focused via keyboard navigation.
    */
-  pspec = g_param_spec_boolean ("can-focus",
-                                "Can focus",
-                                "Whether the widget can be focused via keyboard navigation",
-                                FALSE,
-                                ST_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_CAN_FOCUS,
-                                   pspec);
+  props[PROP_CAN_FOCUS] =
+     g_param_spec_boolean ("can-focus",
+                           "Can focus",
+                           "Whether the widget can be focused via keyboard navigation",
+                           FALSE,
+                           ST_PARAM_READWRITE);
 
   /**
    * ClutterActor:label-actor:
    *
    * An actor that labels this widget.
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_LABEL_ACTOR,
-                                   g_param_spec_object ("label-actor",
-                                                        "Label",
-                                                        "Label that identifies this widget",
-                                                        CLUTTER_TYPE_ACTOR,
-                                                        ST_PARAM_READWRITE));
+  props[PROP_LABEL_ACTOR] =
+     g_param_spec_object ("label-actor",
+                          "Label",
+                          "Label that identifies this widget",
+                          CLUTTER_TYPE_ACTOR,
+                          ST_PARAM_READWRITE);
+
   /**
    * StWidget:accessible-role:
    *
    * The accessible role of this object
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_ACCESSIBLE_ROLE,
-                                   g_param_spec_enum ("accessible-role",
-                                                      "Accessible Role",
-                                                      "The accessible role of this object",
-                                                      ATK_TYPE_ROLE,
-                                                      ATK_ROLE_INVALID,
-                                                      G_PARAM_READWRITE));
-
+  props[PROP_ACCESSIBLE_ROLE] =
+     g_param_spec_enum ("accessible-role",
+                        "Accessible Role",
+                        "The accessible role of this object",
+                        ATK_TYPE_ROLE,
+                        ATK_ROLE_INVALID,
+                        ST_PARAM_READWRITE);
 
   /**
    * StWidget:accessible-name:
    *
    * Object instance's name for assistive technology access.
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_ACCESSIBLE_NAME,
-                                   g_param_spec_string ("accessible-name",
-                                                        "Accessible name",
-                                                        "Object instance's name for assistive technology access.",
-                                                        NULL,
-                                                        ST_PARAM_READWRITE));
+  props[PROP_ACCESSIBLE_NAME] =
+     g_param_spec_string ("accessible-name",
+                          "Accessible name",
+                          "Object instance's name for assistive technology access.",
+                          NULL,
+                          ST_PARAM_READWRITE);
+
+  g_object_class_install_properties (gobject_class, N_PROPS, props);
 
   /**
    * StWidget::style-changed:
@@ -1051,52 +1017,6 @@ st_widget_class_init (StWidgetClass *klass)
                   G_TYPE_NONE, 0);
 }
 
-/**
- * st_widget_set_theme:
- * @actor: a #StWidget
- * @theme: a new style class string
- *
- * Overrides the theme that would be inherited from the actor's parent
- * or the stage with an entirely new theme (set of stylesheets).
- */
-void
-st_widget_set_theme (StWidget  *actor,
-                     StTheme   *theme)
-{
-  StWidgetPrivate *priv;
-
-  g_return_if_fail (ST_IS_WIDGET (actor));
-
-  priv = st_widget_get_instance_private (actor);
-
-  if (theme != priv->theme)
-    {
-      if (priv->theme)
-        g_object_unref (priv->theme);
-      priv->theme = g_object_ref (theme);
-
-      st_widget_style_changed (actor);
-
-      g_object_notify (G_OBJECT (actor), "theme");
-    }
-}
-
-/**
- * st_widget_get_theme:
- * @actor: a #StWidget
- *
- * Gets the overriding theme set on the actor. See st_widget_set_theme()
- *
- * Return value: (transfer none): the overriding theme, or %NULL
- */
-StTheme *
-st_widget_get_theme (StWidget *actor)
-{
-  g_return_val_if_fail (ST_IS_WIDGET (actor), NULL);
-
-  return ST_WIDGET_PRIVATE (actor)->theme;
-}
-
 static const gchar *
 find_class_name (const gchar *class_list,
                  const gchar *class_name)
@@ -1213,7 +1133,7 @@ st_widget_set_style_class_name (StWidget    *actor,
   if (set_class_list (&priv->style_class, style_class_list))
     {
       st_widget_style_changed (actor);
-      g_object_notify (G_OBJECT (actor), "style-class");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_STYLE_CLASS]);
     }
 }
 
@@ -1239,7 +1159,7 @@ st_widget_add_style_class_name (StWidget    *actor,
   if (add_class_name (&priv->style_class, style_class))
     {
       st_widget_style_changed (actor);
-      g_object_notify (G_OBJECT (actor), "style-class");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_STYLE_CLASS]);
     }
 }
 
@@ -1265,7 +1185,7 @@ st_widget_remove_style_class_name (StWidget    *actor,
   if (remove_class_name (&priv->style_class, style_class))
     {
       st_widget_style_changed (actor);
-      g_object_notify (G_OBJECT (actor), "style-class");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_STYLE_CLASS]);
     }
 }
 
@@ -1376,7 +1296,7 @@ st_widget_set_style_pseudo_class (StWidget    *actor,
   if (set_class_list (&priv->pseudo_class, pseudo_class_list))
     {
       st_widget_style_changed (actor);
-      g_object_notify (G_OBJECT (actor), "pseudo-class");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_PSEUDO_CLASS]);
     }
 }
 
@@ -1402,7 +1322,7 @@ st_widget_add_style_pseudo_class (StWidget    *actor,
   if (add_class_name (&priv->pseudo_class, pseudo_class))
     {
       st_widget_style_changed (actor);
-      g_object_notify (G_OBJECT (actor), "pseudo-class");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_PSEUDO_CLASS]);
     }
 }
 
@@ -1427,7 +1347,7 @@ st_widget_remove_style_pseudo_class (StWidget    *actor,
   if (remove_class_name (&priv->pseudo_class, pseudo_class))
     {
       st_widget_style_changed (actor);
-      g_object_notify (G_OBJECT (actor), "pseudo-class");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_PSEUDO_CLASS]);
     }
 }
 
@@ -1457,7 +1377,7 @@ st_widget_set_style (StWidget  *actor,
 
       st_widget_style_changed (actor);
 
-      g_object_notify (G_OBJECT (actor), "style");
+      g_object_notify_by_pspec (G_OBJECT (actor), props[PROP_STYLE]);
     }
 }
 
@@ -1907,7 +1827,7 @@ st_widget_set_track_hover (StWidget *widget,
   if (priv->track_hover != track_hover)
     {
       priv->track_hover = track_hover;
-      g_object_notify (G_OBJECT (widget), "track-hover");
+      g_object_notify_by_pspec (G_OBJECT (widget), props[PROP_TRACK_HOVER]);
 
       if (priv->track_hover)
         st_widget_sync_hover (widget);
@@ -1962,7 +1882,7 @@ st_widget_set_hover (StWidget *widget,
         st_widget_add_style_pseudo_class (widget, "hover");
       else
         st_widget_remove_style_pseudo_class (widget, "hover");
-      g_object_notify (G_OBJECT (widget), "hover");
+      g_object_notify_by_pspec (G_OBJECT (widget), props[PROP_HOVER]);
     }
 }
 
@@ -2029,7 +1949,7 @@ st_widget_set_can_focus (StWidget *widget,
   if (priv->can_focus != can_focus)
     {
       priv->can_focus = can_focus;
-      g_object_notify (G_OBJECT (widget), "can-focus");
+      g_object_notify_by_pspec (G_OBJECT (widget), props[PROP_CAN_FOCUS]);
     }
 }
 
@@ -2071,7 +1991,7 @@ filter_by_position (GList            *children,
                     StDirectionType   direction)
 {
   ClutterActorBox cbox;
-  ClutterVertex abs_vertices[4];
+  graphene_point3d_t abs_vertices[4];
   GList *l, *ret;
   ClutterActor *child;
 
@@ -2137,7 +2057,7 @@ get_distance (ClutterActor    *actor,
 {
   int ax, ay, bx, by, dx, dy;
   ClutterActorBox abox;
-  ClutterVertex abs_vertices[4];
+  graphene_point3d_t abs_vertices[4];
 
   clutter_actor_get_abs_allocation_vertices (actor, abs_vertices);
   clutter_actor_box_from_vertices (&abox, abs_vertices);
@@ -2241,7 +2161,7 @@ st_widget_real_navigate_focus (StWidget         *widget,
   else /* direction is an arrow key, not tab */
     {
       ClutterActorBox sort_box;
-      ClutterVertex abs_vertices[4];
+      graphene_point3d_t abs_vertices[4];
 
       /* Compute the allocation box of the previous focused actor. If there
        * was no previous focus, use the coordinates of the appropriate edge of
@@ -2456,30 +2376,6 @@ st_describe_actor (ClutterActor *actor)
   return g_string_free (desc, FALSE);
 }
 
-/**
- * st_set_slow_down_factor:
- * @factor: new slow-down factor
- *
- * Set a global factor applied to all animation durations
- */
-void
-st_set_slow_down_factor (gfloat factor)
-{
-  st_slow_down_factor = factor;
-}
-
-/**
- * st_get_slow_down_factor:
- *
- * Returns: the global factor applied to all animation durations
- */
-gfloat
-st_get_slow_down_factor (void)
-{
-  return st_slow_down_factor;
-}
-
-
 /**
  * st_widget_get_label_actor:
  * @widget: a #StWidget
@@ -2526,7 +2422,7 @@ st_widget_set_label_actor (StWidget     *widget,
       else
         priv->label_actor = NULL;
 
-      g_object_notify (G_OBJECT (widget), "label-actor");
+      g_object_notify_by_pspec (G_OBJECT (widget), props[PROP_LABEL_ACTOR]);
     }
 }
 
@@ -2559,7 +2455,7 @@ st_widget_set_accessible_name (StWidget    *widget,
     g_free (priv->accessible_name);
 
   priv->accessible_name = g_strdup (name);
-  g_object_notify (G_OBJECT (widget), "accessible-name");
+  g_object_notify_by_pspec (G_OBJECT (widget), props[PROP_ACCESSIBLE_NAME]);
 }
 
 /**
@@ -2613,7 +2509,7 @@ st_widget_set_accessible_role (StWidget *widget,
   priv = st_widget_get_instance_private (widget);
   priv->accessible_role = role;
 
-  g_object_notify (G_OBJECT (widget), "accessible-role");
+  g_object_notify_by_pspec (G_OBJECT (widget), props[PROP_ACCESSIBLE_ROLE]);
 }
 
 
@@ -3145,63 +3041,3 @@ st_widget_get_focus_chain (StWidget *widget)
 {
   return ST_WIDGET_GET_CLASS (widget)->get_focus_chain (widget);
 }
-
-/**
- * st_get_align_factors:
- * @x_align: an #StAlign
- * @y_align: an #StAlign
- * @x_align_out: (out) (optional): @x_align as a #gdouble
- * @y_align_out: (out) (optional): @y_align as a #gdouble
- *
- * Converts @x_align and @y_align to #gdouble values.
- */
-void
-st_get_align_factors (StAlign   x_align,
-                      StAlign   y_align,
-                      gdouble  *x_align_out,
-                      gdouble  *y_align_out)
-{
-  if (x_align_out)
-    {
-      switch (x_align)
-        {
-        case ST_ALIGN_START:
-          *x_align_out = 0.0;
-          break;
-
-        case ST_ALIGN_MIDDLE:
-          *x_align_out = 0.5;
-          break;
-
-        case ST_ALIGN_END:
-          *x_align_out = 1.0;
-          break;
-
-        default:
-          g_warn_if_reached ();
-          break;
-        }
-    }
-
-  if (y_align_out)
-    {
-      switch (y_align)
-        {
-        case ST_ALIGN_START:
-          *y_align_out = 0.0;
-          break;
-
-        case ST_ALIGN_MIDDLE:
-          *y_align_out = 0.5;
-          break;
-
-        case ST_ALIGN_END:
-          *y_align_out = 1.0;
-          break;
-
-        default:
-          g_warn_if_reached ();
-          break;
-        }
-    }
-}
